@@ -18,6 +18,7 @@
  */
 #include "ClientConnection.h"
 
+#include <openssl/x509.h>
 #include <pulsar/MessageIdBuilder.h>
 
 #include <boost/optional.hpp>
@@ -50,8 +51,6 @@ using proto::BaseCommand;
 
 static const uint32_t DefaultBufferSize = 64 * 1024;
 
-static const int KeepAliveIntervalInSeconds = 30;
-
 static MessageId toMessageId(const proto::MessageIdData& messageIdData) {
     return MessageIdBuilder::from(messageIdData).build();
 }
@@ -171,7 +170,13 @@ ClientConnection::ClientConnection(const std::string& logicalAddress, const std:
       executor_(executor),
       resolver_(executor_->createTcpResolver()),
       socket_(executor_->createSocket()),
+#if defined(USE_ASIO) || BOOST_VERSION >= 107000
       strand_(ASIO::make_strand(executor_->getIOService().get_executor())),
+#elif BOOST_VERSION >= 106600
+      strand_(executor_->getIOService().get_executor()),
+#else
+      strand_(executor_->getIOService()),
+#endif
       logicalAddress_(logicalAddress),
       physicalAddress_(physicalAddress),
       cnxString_("[<none> -> " + physicalAddress + "] "),
@@ -179,6 +184,7 @@ ClientConnection::ClientConnection(const std::string& logicalAddress, const std:
       connectTimeoutTask_(
           std::make_shared<PeriodicTask>(*executor_, clientConfiguration.getConnectionTimeout())),
       outgoingBuffer_(SharedBuffer::allocate(DefaultBufferSize)),
+      keepAliveIntervalInSeconds_(clientConfiguration.getKeepAliveIntervalInSeconds()),
       consumerStatsRequestTimer_(executor_->createDeadlineTimer()),
       maxPendingLookupRequest_(clientConfiguration.getConcurrentLookupRequest()),
       clientVersion_(clientVersion),
@@ -227,6 +233,7 @@ ClientConnection::ClientConnection(const std::string& logicalAddress, const std:
                 }
             } else {
                 ctx.set_default_verify_paths();
+                LOG_INFO("Use " << X509_get_default_cert_file() << " as default CA path");
             }
         }
 
@@ -246,11 +253,11 @@ ClientConnection::ClientConnection(const std::string& logicalAddress, const std:
                 throw ResultAuthenticationError;
             }
             ctx.use_private_key_file(tlsPrivateKey, ASIO::ssl::context::pem);
-            ctx.use_certificate_file(tlsCertificates, ASIO::ssl::context::pem);
+            ctx.use_certificate_chain_file(tlsCertificates);
         } else {
             if (file_exists(tlsPrivateKey) && file_exists(tlsCertificates)) {
                 ctx.use_private_key_file(tlsPrivateKey, ASIO::ssl::context::pem);
-                ctx.use_certificate_file(tlsCertificates, ASIO::ssl::context::pem);
+                ctx.use_certificate_chain_file(tlsCertificates);
             }
         }
 
@@ -302,7 +309,7 @@ void ClientConnection::handlePulsarConnected(const proto::CommandConnected& cmdC
         // Only send keep-alive probes if the broker supports it
         keepAliveTimer_ = executor_->createDeadlineTimer();
         if (keepAliveTimer_) {
-            keepAliveTimer_->expires_from_now(std::chrono::seconds(KeepAliveIntervalInSeconds));
+            keepAliveTimer_->expires_from_now(std::chrono::seconds(keepAliveIntervalInSeconds_));
             auto weakSelf = weak_from_this();
             keepAliveTimer_->async_wait([weakSelf](const ASIO_ERROR&) {
                 auto self = weakSelf.lock();
@@ -521,8 +528,15 @@ void ClientConnection::handleHandshake(const ASIO_ERROR& err) {
 
     bool connectingThroughProxy = logicalAddress_ != physicalAddress_;
     Result result = ResultOk;
-    SharedBuffer buffer = Commands::newConnect(authentication_, logicalAddress_, connectingThroughProxy,
-                                               clientVersion_, result);
+    SharedBuffer buffer;
+    try {
+        buffer = Commands::newConnect(authentication_, logicalAddress_, connectingThroughProxy,
+                                      clientVersion_, result);
+    } catch (const std::exception& e) {
+        LOG_ERROR(cnxString_ << "Failed to create Connect command: " << e.what());
+        close(ResultAuthenticationError);
+        return;
+    }
     if (result != ResultOk) {
         LOG_ERROR(cnxString_ << "Failed to establish connection: " << result);
         close(result);
@@ -1237,7 +1251,7 @@ void ClientConnection::handleKeepAliveTimeout() {
         // be zero And we do not attempt to dereference the pointer.
         Lock lock(mutex_);
         if (keepAliveTimer_) {
-            keepAliveTimer_->expires_from_now(std::chrono::seconds(KeepAliveIntervalInSeconds));
+            keepAliveTimer_->expires_from_now(std::chrono::seconds(keepAliveIntervalInSeconds_));
             auto weakSelf = weak_from_this();
             keepAliveTimer_->async_wait([weakSelf](const ASIO_ERROR&) {
                 auto self = weakSelf.lock();
